#include "BF537 Flags.h"

//--------------------------------------------------------------------------//
// Function:	Timer2_ISR						
//																
// Parameters:	None												
//																	
// Return:		None												
//																		
// Description:	This routine serves as a debug aid when the TIMER2 IRQ is enabled.
//              When the TIMER2 IRQ is enabled, this routine will corrupt FFT
//              results for VEC_SIZE greater than 256 (FFT routine uses r5,r6, 
//              and r7).
//--------------------------------------------------------------------------//

EX_INTERRUPT_HANDLER(Timer2_ISR)
{ 
    
    // confirm interrupt handling
	*pTIMER_STATUS = 0x0004;    //write one to clear timer 2 interrupt
	
    Int2Cnt += 1;               //global  
    
    if(Int2Cnt == 1)
    {
       asm("r5=0;"); 
       asm("cycles = r5;");
    }
    
    if(Int2Cnt == 2)
    {
       asm("r6=cycles;");
       asm("r7=r6-r5;");        //Determine timer2 rate from cycle counter
    }   
    
}


EX_INTERRUPT_HANDLER(Mem0StreamDMA_ISR)
{

        //*pMDMA_S0_CONFIG = WDSIZE_16; //clear DMA_EN
        //*pMDMA_D0_CONFIG = WNR | WDSIZE_16 | FLOW_STOP | DI_EN;
        ssync();
}

//--------------------------------------------------------------------------//
// Function:	PORTF_IntA_ISR			
//
//													
// Parameters:	None										
//														
// Return:		None								
//														
// Description:	Sets operating mode                                   
//--------------------------------------------------------------------------//

EX_INTERRUPT_HANDLER(PORTF_IntA_ISR)
{
    
    int temp_fio;
    
    temp_fio = *pPORTFIO & 0x0024;
    
    if (temp_fio == 0x0004)
    	{OpMode = ONE_SHOT;}    //BF537 EZ-KIT PB1
    else
    {
    	if (temp_fio == 0x0020) //BF537 EZ-KIT PB5
    	{OpMode = BLOCK;}
    }
    
	// confirm interrupt handling and clear flags
	*pPORTFIO_CLEAR = 0x0024;

}

EX_INTERRUPT_HANDLER(PORTF_IntB_ISR) //PF8 falling edge - AD7656 BUSY signal
{
    
    int temp_busy;
    int temp;
    int j;
    
    temp_busy = *pPORTFIO & 0x0100;  //mask for BUSY bit
    
	// confirm interrupt handling and clear flags
	*pPORTFIO_CLEAR = 0x0100;
	
	if(temp_busy)
	{
	    //Initiate DMA read
    
    	temp = AD7656IntCnt*CHAN_CNT;     //see BF537Flags.h
    
    	if(OpMode == BLOCK)
    	{
    		if(AD7656IntCnt < VEC_SIZE)   //see BF537Flags.h
        	{

	       		*pMDMA_S0_START_ADDR = (void *)(AD7656_address);
	       		*pMDMA_S0_X_COUNT    = CHAN_CNT;
	       		*pMDMA_S0_X_MODIFY   = 2;
           		*pMDMA_S0_CONFIG     = WDSIZE_16 | DMAEN;
           		ssync();
    		
           		*pMDMA_D0_START_ADDR = AD7656_input + temp;
	       		*pMDMA_D0_X_COUNT    = CHAN_CNT;
	       		*pMDMA_D0_X_MODIFY   = 2;
           		//*pMDMA_D0_CONFIG     = WNR | WDSIZE_16 | FLOW_STOP | DI_EN | DMAEN;
           		*pMDMA_D0_CONFIG     = WNR | WDSIZE_16 | FLOW_STOP | DMAEN;
           		ssync();
       
           		AD7656IntCnt += 1;
        	}
        	else                      //AD7656IntCnt = VEC_SIZE
        	{

            	AD7656IntCnt = 0;     //VEC_SIZE * 6 samples received
            	RunFFT = 1;           //Run FFT on channel 1 data
        	}  
       
    	}
    	else                          //OpMode = ONE_SHOT
    	{
    	    

	         for(j=0; j<100; j++) 
	             { 
	                 asm("nop;");
	             }
	   
    	    
    	    
           *pMDMA_S0_START_ADDR = (void *)(AD7656_address);
	       *pMDMA_S0_X_COUNT    = CHAN_CNT;
	       *pMDMA_S0_X_MODIFY   = 2;
           *pMDMA_S0_CONFIG     = WDSIZE_16 | DMAEN;
           ssync();
    
           *pMDMA_D0_START_ADDR = (void *)AD7656_input;
	       *pMDMA_D0_X_COUNT    = CHAN_CNT;
	       *pMDMA_D0_X_MODIFY   = 2;
           //*pMDMA_D0_CONFIG     = WNR | WDSIZE_16 | FLOW_STOP | DI_EN | DMAEN;
           *pMDMA_D0_CONFIG     = WNR | WDSIZE_16 | FLOW_STOP | DMAEN;
           ssync();
           
    	}
	}      //close bracket for: if(temp_busy)
}

